@@ -28,6 +28,7 @@ const Notifications = imports.notifications;
 const Password = imports.password;
 const Preview = imports.preview;
 const Edit = imports.edit;
+const Search = imports.search;
 const Selections = imports.selections;
 const View = imports.view;
 const WindowMode = imports.windowMode;
@@ -73,136 +74,6 @@ const SpinnerBox = new Lang.Class({
     }
 });
 
-const ErrorBox = new Lang.Class({
-    Name: 'ErrorBox',
-
-    _init: function(primary, secondary) {
-        this.widget = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
-                                     row_spacing: 12,
-                                     hexpand: true,
-                                     vexpand: true,
-                                     halign: Gtk.Align.CENTER,
-                                     valign: Gtk.Align.CENTER });
-
-        this._image = new Gtk.Image({ pixel_size: _ICON_SIZE,
-                                      icon_name: 'face-uncertain-symbolic',
-                                      halign: Gtk.Align.CENTER,
-                                      valign: Gtk.Align.CENTER });
-
-        this.widget.add(this._image);
-
-        this._primaryLabel =
-            new Gtk.Label({ label: '',
-                            use_markup: true,
-                            halign: Gtk.Align.CENTER,
-                            valign: Gtk.Align.CENTER });
-        this.widget.add(this._primaryLabel);
-
-        this._secondaryLabel =
-            new Gtk.Label({ label: '',
-                            use_markup: true,
-                            wrap: true,
-                            halign: Gtk.Align.CENTER,
-                            valign: Gtk.Align.CENTER });
-        this.widget.add(this._secondaryLabel);
-
-        this.widget.show_all();
-    },
-
-    update: function(primary, secondary) {
-        let primaryMarkup = '<big><b>' + GLib.markup_escape_text(primary, -1) + '</b></big>';
-        let secondaryMarkup = GLib.markup_escape_text(secondary, -1);
-
-        this._primaryLabel.label = primaryMarkup;
-        this._secondaryLabel.label = secondaryMarkup;
-    }
-});
-
-const EmptyResultsBox = new Lang.Class({
-    Name: 'EmptyResultsBox',
-
-    _init: function() {
-        this.widget = new Gtk.Grid({ orientation: Gtk.Orientation.HORIZONTAL,
-                                     column_spacing: 12,
-                                     hexpand: true,
-                                     vexpand: true,
-                                     halign: Gtk.Align.CENTER,
-                                     valign: Gtk.Align.CENTER });
-        this.widget.get_style_context().add_class('dim-label');
-
-        this._image = new Gtk.Image({ pixel_size: 64,
-                                      icon_name: 'emblem-documents-symbolic' });
-        this.widget.add(this._image);
-
-        this._labelsGrid = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
-                                          row_spacing: 12 });
-        this.widget.add(this._labelsGrid);
-
-        let titleLabel = new Gtk.Label({ label: '<b><span size="large">' +
-                                         (Application.application.isBooks ?
-                                          _("No Books Found") :
-                                          _("No Documents Found")) +
-                                         '</span></b>',
-                                         use_markup: true,
-                                         halign: Gtk.Align.START,
-                                         vexpand: true });
-        this._labelsGrid.add(titleLabel);
-
-        if (Application.sourceManager.hasOnlineSources() ||
-            Application.application.isBooks) {
-            titleLabel.valign = Gtk.Align.CENTER;
-        } else {
-            titleLabel.valign = Gtk.Align.START;
-            this._addSystemSettingsLabel();
-        }
-
-        this.widget.show_all();
-    },
-
-    _addSystemSettingsLabel: function() {
-        let detailsStr =
-            // Translators: %s here is "Settings", which is in a separate string due to
-            // markup, and should be translated only in the context of this sentence
-            _("You can add your online accounts in %s").format(
-            " <a href=\"system-settings\">" +
-            // Translators: this should be translated in the context of the
-            // "You can add your online accounts in Settings" sentence above
-            _("Settings") +
-            "</a>");
-        let details = new Gtk.Label({ label: detailsStr,
-                                      use_markup: true,
-                                      halign: Gtk.Align.START,
-                                      xalign: 0,
-                                      max_width_chars: 24,
-                                      wrap: true });
-        this._labelsGrid.add(details);
-
-        details.connect('activate-link', Lang.bind(this,
-            function(label, uri) {
-                if (uri != 'system-settings')
-                    return false;
-
-                try {
-                    let app = Gio.AppInfo.create_from_commandline(
-                        'gnome-control-center online-accounts', null, 0);
-
-                    let screen = this.widget.get_screen();
-                    let display = screen ? screen.get_display() : Gdk.Display.get_default();
-                    let ctx = display.get_app_launch_context();
-
-                    if (screen)
-                        ctx.set_screen(screen);
-
-                    app.launch([], ctx);
-                } catch(e) {
-                    log('Unable to launch gnome-control-center: ' + e.message);
-                }
-
-                return true;
-            }));
-    }
-});
-
 const Embed = new Lang.Class({
     Name: 'Embed',
 
@@ -234,8 +105,14 @@ const Embed = new Lang.Class({
         this._stackOverlay.add_overlay(Application.notificationManager.widget);
 
         // now create the actual content widgets
-        this._view = new View.ViewContainer();
-        this._stack.add_named(this._view.widget, 'view');
+        this._documents = new View.ViewContainer(WindowMode.WindowMode.DOCUMENTS);
+        this._stack.add_titled(this._documents.widget, 'documents', _("Recent"));
+
+        this._collections = new View.ViewContainer(WindowMode.WindowMode.COLLECTIONS);
+        this._stack.add_titled(this._collections.widget, 'collections', _("Collections"));
+
+        this._search = new View.ViewContainer(WindowMode.WindowMode.SEARCH);
+        this._stack.add_named(this._search.widget, 'search');
 
         this._preview = new Preview.PreviewView(this._stackOverlay);
         this._stack.add_named(this._preview.widget, 'preview');
@@ -246,24 +123,16 @@ const Embed = new Lang.Class({
         this._spinnerBox = new SpinnerBox();
         this._stack.add_named(this._spinnerBox.widget, 'spinner');
 
-        this._errorBox = new ErrorBox();
-        this._stack.add_named(this._errorBox.widget, 'error');
-
-        this._noResults = new EmptyResultsBox();
-        this._stack.add_named(this._noResults.widget, 'no-results');
+        this._stack.connect('notify::visible-child',
+                            Lang.bind(this, this._onVisibleChildChanged));
 
         Application.modeController.connect('window-mode-changed',
                                            Lang.bind(this, this._onWindowModeChanged));
 
         Application.modeController.connect('fullscreen-changed',
                                            Lang.bind(this, this._onFullscreenChanged));
-        Application.trackerController.connect('query-status-changed',
-                                              Lang.bind(this, this._onQueryStatusChanged));
-        Application.trackerController.connect('query-error',
-                                              Lang.bind(this, this._onQueryError));
-
-        Application.offsetController.connect('item-count-changed',
-                                             Lang.bind(this, this._onItemCountChanged));
+        Application.trackerDocumentsController.connect('query-status-changed',
+                                                       Lang.bind(this, this._onQueryStatusChanged));
 
         Application.documentManager.connect('active-changed',
                                             Lang.bind(this, this._onActiveItemChanged));
@@ -276,6 +145,14 @@ const Embed = new Lang.Class({
         Application.documentManager.connect('password-needed',
                                             Lang.bind(this, this._onPasswordNeeded));
 
+        Application.searchTypeManager.connect('active-changed',
+                                              Lang.bind(this, this._onSearchChanged));
+        Application.sourceManager.connect('active-changed',
+                                          Lang.bind(this, this._onSearchChanged));
+
+        Application.searchController.connect('search-string-changed',
+                                             Lang.bind(this, this._onSearchChanged));
+
         this._onQueryStatusChanged();
 
         let windowMode = Application.modeController.getWindowMode();
@@ -283,74 +160,130 @@ const Embed = new Lang.Class({
             this._onWindowModeChanged(Application.modeController, windowMode, WindowMode.WindowMode.NONE);
     },
 
+    _getViewFromMode: function(windowMode) {
+        let view;
+
+        switch (windowMode) {
+        case WindowMode.WindowMode.COLLECTIONS:
+            view = this._collections;
+            break;
+        case WindowMode.WindowMode.DOCUMENTS:
+            view = this._documents;
+            break;
+        case WindowMode.WindowMode.PREVIEW:
+            view = this._preview;
+            break;
+        case WindowMode.WindowMode.SEARCH:
+            view = this._search;
+            break;
+        default:
+            throw(new Error('Not handled'));
+            break;
+        }
+
+        return view;
+    },
+
     _onActivateResult: function() {
         let windowMode = Application.modeController.getWindowMode();
+        let view = this._getViewFromMode(mode);
+        view.activateResult();
+    },
 
-        if (windowMode == WindowMode.WindowMode.OVERVIEW)
-            this._view.activateResult();
-        else if (windowMode == WindowMode.WindowMode.PREVIEW)
-            this._preview.activateResult();
+    _restoreLastPage: function() {
+        let page;
+        let windowMode = Application.modeController.getWindowMode();
+
+        switch (windowMode) {
+        case WindowMode.WindowMode.COLLECTIONS:
+            page = 'collections';
+            break;
+        case WindowMode.WindowMode.DOCUMENTS:
+            page = 'documents';
+            break;
+        case WindowMode.WindowMode.PREVIEW:
+            page = 'preview';
+            break;
+        case WindowMode.WindowMode.SEARCH:
+            page = 'search';
+            break;
+        default:
+            throw(new Error('Not handled'));
+            break;
+        }
+
+        this._stack.set_visible_child_name(page);
     },
 
     _onQueryStatusChanged: function() {
-        let windowMode = Application.modeController.getWindowMode();
-        if (windowMode != WindowMode.WindowMode.OVERVIEW)
-            return;
-
-        let queryStatus = Application.trackerController.getQueryStatus();
+        let queryStatus = Application.trackerDocumentsController.getQueryStatus();
 
         if (queryStatus) {
             this._spinnerBox.start();
             this._stack.set_visible_child_name('spinner');
         } else {
             this._spinnerBox.stop();
-            this._stack.set_visible_child_name('view');
+            this._restoreLastPage();
         }
     },
 
-    _hideNoResultsPage: function() {
-        if (this._noResultsChangeId != 0) {
-            Application.changeMonitor.disconnect(this._noResultsChangeId);
-            this._noResultsChangeId = 0;
-        }
-
-        this._stack.set_visible_child_name('view');
-    },
-
-    _onItemCountChanged: function() {
-        let windowMode = Application.modeController.getWindowMode();
-        if (windowMode != WindowMode.WindowMode.OVERVIEW)
-            return;
-
-        let itemCount = Application.offsetController.getItemCount();
-
-        if (itemCount == 0) {
-            // also listen to changes-pending while in this mode
-            this._noResultsChangeId =
-                Application.changeMonitor.connect('changes-pending', Lang.bind(this,
-                    function() {
-                        this._hideNoResultsPage();
-                    }));
-
-            this._stack.set_visible_child_name('no-results');
-        } else {
-            this._hideNoResultsPage();
-        }
-    },
-
-    _onQueryError: function(manager, message, exception) {
-        this._setError(message, exception.message);
-    },
-
     _onFullscreenChanged: function(controller, fullscreen) {
         this._toolbar.widget.visible = !fullscreen;
         this._toolbar.widget.sensitive = !fullscreen;
     },
 
+    _onSearchChanged: function() {
+        // Whenever a search constraint is specified we want to switch to
+        // the search mode, and when all constraints have been lifted we
+        // want to go back to the previous mode which can be either
+        // collections or documents.
+        //
+        // However there are some exceptions, which are taken care of
+        // elsewhere:
+        //  - when moving from search to preview or collection view
+        //  - when in preview or coming out of it
+
+        let doc = Application.documentManager.getActiveItem();
+        let windowMode = Application.modeController.getWindowMode();
+        if (windowMode == WindowMode.WindowMode.SEARCH && doc)
+            return;
+        if (windowMode == WindowMode.WindowMode.PREVIEW)
+            return;
+
+        let searchType = Application.searchTypeManager.getActiveItem();
+        let source = Application.sourceManager.getActiveItem();
+        let str = Application.searchController.getString();
+
+        if (searchType.id == Search.SearchTypeStock.ALL &&
+            source.id == Search.SearchSourceStock.ALL &&
+            (!str || str == '')) {
+            Application.modeController.goBack();
+        } else {
+            Application.modeController.setWindowMode(WindowMode.WindowMode.SEARCH);
+        }
+    },
+
+    _onVisibleChildChanged: function() {
+        let visibleChild = this._stack.visible_child;
+        let windowMode = WindowMode.WindowMode.NONE;
+
+        if (visibleChild == this._collections.widget)
+            windowMode = WindowMode.WindowMode.COLLECTIONS;
+        else if (visibleChild == this._documents.widget)
+            windowMode = WindowMode.WindowMode.DOCUMENTS;
+
+        if (windowMode == WindowMode.WindowMode.NONE)
+            return;
+
+        Application.modeController.setWindowMode(windowMode);
+    },
+
     _onWindowModeChanged: function(object, newMode, oldMode) {
         switch (newMode) {
-        case WindowMode.WindowMode.OVERVIEW:
-            this._prepareForOverview();
+        case WindowMode.WindowMode.COLLECTIONS:
+        case WindowMode.WindowMode.DOCUMENTS:
+        case WindowMode.WindowMode.SEARCH:
+            this._prepareForOverview(newMode, oldMode);
             break;
         case WindowMode.WindowMode.PREVIEW:
             if (oldMode == WindowMode.WindowMode.EDIT)
@@ -374,13 +307,6 @@ const Embed = new Lang.Class({
     },
 
     _onActiveItemChanged: function(manager, doc) {
-        if (doc) {
-            let collection = Application.collectionManager.getItemById(doc.id);
-            if (!collection)
-                return;
-        }
-
-        Application.modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
     },
 
     _clearLoadTimer: function() {
@@ -422,7 +348,6 @@ const Embed = new Lang.Class({
     _onLoadError: function(manager, doc, message, exception) {
         this._clearLoadTimer();
         this._spinnerBox.stop();
-        this._setError(message, exception.message);
     },
 
     _onPasswordNeeded: function(manager, doc) {
@@ -438,20 +363,44 @@ const Embed = new Lang.Class({
             }));
     },
 
-    _prepareForOverview: function() {
+    _prepareForOverview: function(newMode, oldMode) {
+        let createToolbar = (oldMode != WindowMode.WindowMode.COLLECTIONS &&
+                             oldMode != WindowMode.WindowMode.DOCUMENTS &&
+                             oldMode != WindowMode.WindowMode.SEARCH);
+
+        let visibleChildName;
+
+        switch (newMode) {
+        case WindowMode.WindowMode.COLLECTIONS:
+            visibleChildName = 'collections';
+            break;
+        case WindowMode.WindowMode.DOCUMENTS:
+            visibleChildName = 'documents';
+            break;
+        case WindowMode.WindowMode.SEARCH:
+            visibleChildName = 'search';
+            break;
+        default:
+            throw(new Error('Not handled'));
+            break;
+        }
+
         if (this._preview)
             this._preview.reset();
         if (this._edit)
             this._edit.setUri(null);
-        if (this._toolbar)
-            this._toolbar.widget.destroy();
 
-        // pack the toolbar
-        this._toolbar = new MainToolbar.OverviewToolbar(this._stackOverlay);
-        this._titlebar.add(this._toolbar.widget);
+        if (createToolbar) {
+            if (this._toolbar)
+                this._toolbar.widget.destroy();
+
+            // pack the toolbar
+            this._toolbar = new MainToolbar.OverviewToolbar(this._stackOverlay, this._stack);
+            this._titlebar.add(this._toolbar.widget);
+        }
 
         this._spinnerBox.stop();
-        this._stack.set_visible_child_name('view');
+        this._stack.set_visible_child_name(visibleChildName);
     },
 
     _prepareForPreview: function() {
@@ -480,11 +429,6 @@ const Embed = new Lang.Class({
         this._stack.set_visible_child_name('edit');
     },
 
-    _setError: function(primary, secondary) {
-        this._errorBox.update(primary, secondary);
-        this._stack.set_visible_child_name('error');
-    },
-
     getMainToolbar: function() {
         let windowMode = Application.modeController.getWindowMode();
         let fullscreen = Application.modeController.getFullscreen();
